#!/usr/bin/perl

######################################################################
package Cognition::uF::RelTag;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Cognition::MagicString;
use Digest::SHA1 qw(sha1_hex);

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub consume
{
	my $page   = shift;

	if ( $page->{Profiles}->{'http://purl.org/uF/rel-tag/1.0/'}
	||   $page->{Profiles}->{'http://purl.org/uF/2008/03/'}
	||  !$page->{options}->{uf_strict_profiles})
	{	
		$page->{uF}->{Tag} = [ Cognition::uF::RelTag::parse_all($page) ];
		foreach my $tag (@{ $page->{uF}->{Tag} })
		{
			$page->mergeSubject( $tag->rdf_subject );
			my $val = Cognition::RDFModel::Value->new($tag->uri, 1);
			my $url = $page->abs_url;
			$page->{RDF}->{$url}->addVal('http://microformats.org/wiki/rel-tag#tag', $val);
		}
		if (defined $page->{uF}->{Tag}->[0])
			{ $page->mergeNS( $page->{uF}->{Tag}->[0] ); }
	}
}


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes = Cognition::HTMLParser::Utils::searchRel('tag', $within);
	my @rv;
	
	foreach my $t (@nodes)
	{
		my $t_parsed = parse($page, $t);
		push @rv, $t_parsed;
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	push @rv, 'tag:tagspace.invalid,1999-06-01:' . CGI::Util::escape($this->{tag});
	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	# push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub metadata_ns
{
	my $this = shift;
	return { 'tag' => { nsurl=>'http://microformats.org/wiki/rel-tag#',title=>'rel-tag' } };
} #/sub namespace


sub rdf_subject
{
	my $this    = shift;

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
		Cognition::RDFModel::Value->new('http://microformats.org/wiki/rel-tag#Tag', 1));

	$subject->addVal('http://www.w3.org/2000/01/rdf-schema#label',
		Cognition::RDFModel::Value->new($this->{label}))
		if (length $this->{label});

	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#value',
		Cognition::RDFModel::Value->new($this->{tag}))
		if (length $this->{tag});

	$subject->addVal('http://microformats.org/wiki/rel-tag#tagSpace',
		Cognition::RDFModel::Value->new($this->{tagspace}, 1));

	$subject->addVal("http://www.w3.org/1999/02/22-rdf-syntax-ns#seeAlso",
		Cognition::RDFModel::Value->new($this->{tagspace}.$this->{tag}, 1));
	
	return $subject;
	
} #/sub rdf_subject


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $t_node = $rv->{'_dom'}->cloneNode(1);

	my $id = $t_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	my $ts;
	my $tag = $page->abs_url($t_node->getAttribute('href'));
	$tag =~ s/\#.*$//;
	$tag =~ s/\?.*$//;
	$tag =~ s/\/$//;
	
	if ($tag =~ m{^(.*/)([^/]+)$})
	{
		$rv->{tagspace} = $1;
		$rv->{tag}      = ms CGI::Util::unescape($2), $t_node;
	}
	
	$rv->{label} = Cognition::HTMLParser::Utils::STRINGIFY($t_node, 'value');
	$rv->{title} = (length $t_node->getAttribute('title'))
		? $t_node->getAttribute('title')
		: $rv->{label};
	
	bless $rv;
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::RelTag




